@@ -8,5 +8,5 @@ idr library
 name = "midr"
 __all__ = ["__main__",
     "idr", "samic", "archimedean", "c_archimedean", "archimediean_plots",
-    "log", "narrowpeak"]
+    "log", "narrowpeak", "raw_matrix", "auxiliary"]
 
