@@ -13,7 +13,6 @@ call for the merged replicate. This tool computes and appends a IDR column to
 NarrowPeaks files.
 """
 
-import midr.log as log
 from sys import float_info
 import numpy as np
 import functools
@@ -304,7 +303,7 @@ def dmle_copula_frank(u_values):
     )
 
 
-def ipsi_clayton(x, theta, is_log=False):
+def ipsi_clayton(u_values, theta, is_log=False):
     """
     compute Clayton iPsi function
     :param x:
@@ -359,14 +358,14 @@ def ipsi_clayton(x, theta, is_log=False):
            [-1.05452015, -0.85128374, -2.14210478],
            [-2.47465594, -2.58334647, -1.40228078]])
     """
-    x = np.array(x, dtype=np.float64)
+    x = np.array(u_values, dtype=np.float64)
     res = np.sign(theta) * ((x ** -theta) - 1.0)
     if is_log:
         return np.array(np.log(res), dtype=np.float64)
     return np.array(res, dtype=np.float64)
 
 
-def psi_clayton(x, theta):
+def psi_clayton(u_values, theta):
     """
     compute Clayton Psi function
     :param x:
@@ -396,9 +395,122 @@ def psi_clayton(x, theta):
            [0.36343815, 0.45791558, 0.10349543],
            [0.07755952, 0.07150121, 0.23766697]])
     """
-    return np.maximum(1.0 + np.sign(theta) * x, 0.0) ** (-1.0 / theta)
+    return np.maximum(1.0 + np.sign(theta) * u_values, 0.0) ** (-1.0 / theta)
 
 
+def cdf_gumbel(u_values, theta, is_log=False):
+    """
+    compute gumbel copula cdf
+    :param u_values:
+    :param theta:
+    :param is_log:
+    :return:
+    >>> cdf_gumbel(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266],
+    ...    [0.66752741, 0.66752741, 0.66752741]
+    ...    ]),
+    ...    2)
+    array([0.1913089 , 0.4486167 , 0.21746145, 0.35144404, 0.37645985,
+           0.14718965, 0.1466199 , 0.11544204, 0.31594332, 0.13185828,
+           0.07929224])
+    """
+    return ipsi_gumbel(
+        u_values=np.sum(
+            psi_gumbel(
+                u_values=u_values,
+                theta=theta
+            ),
+            axis=1)
+        ,
+        theta=theta,
+        is_log=is_log
+    )
+
+
+def cdf_frank(u_values, theta, is_log=False):
+    """
+    compute frank copula cdf
+    :param u_values:
+    :param theta:
+    :param is_log:
+    :return:
+    >>> cdf_frank(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266],
+    ...    [0.66752741, 0.66752741, 0.66752741]
+    ...    ]),
+    ...    6)
+    array([0.03462875, 0.00121066, 0.0185123 , 0.00254497, 0.00124264,
+           0.06281442, 0.05223709, 0.07652003, 0.01163689, 0.06994171,
+           0.12129838])
+    """
+    return ipsi_frank(
+        u_values=np.sum(
+            psi_frank(
+                u_values=u_values,
+                theta=theta
+            ),
+            axis=1)
+        ,
+        theta=theta,
+        is_log=is_log
+    )
+
+
+def cdf_clayton(u_values, theta, is_log=False):
+    """
+    compute clayton copula cdf
+    :param u_values:
+    :param theta:
+    :param is_log:
+    :return:
+    >>> cdf_clayton(np.array([
+    ...    [0.42873569, 0.18285458, 0.9514195],
+    ...    [0.25148149, 0.05617784, 0.3378213],
+    ...    [0.79410993, 0.76175687, 0.0709562],
+    ...    [0.02694249, 0.45788802, 0.6299574],
+    ...    [0.39522060, 0.02189511, 0.6332237],
+    ...    [0.66878367, 0.38075101, 0.5185625],
+    ...    [0.90365653, 0.19654621, 0.6809525],
+    ...    [0.28607729, 0.82713755, 0.7686878],
+    ...    [0.22437343, 0.16907646, 0.5740400],
+    ...    [0.66752741, 0.69487362, 0.3329266],
+    ...    [0.66752741, 0.66752741, 0.66752741]
+    ...    ]),
+    ...    0.2)
+    array([ 0.09501918, -0.0540091 ,  0.03984838, -0.0214099 , -0.03133615,
+            0.20090686,  0.13875137,  0.20653633,  0.01574763,  0.20925793,
+            0.33859356])
+    """
+    return ipsi_clayton(
+        u_values=np.sum(
+            psi_clayton(
+                u_values=u_values,
+                theta=theta
+            ),
+            axis=1)
+        ,
+        theta=theta,
+        is_log=is_log
+    )
+
 def pdf_clayton(u_values, theta, is_log=False):
     """
     compute clayton copula pdf
@@ -1236,9 +1348,9 @@ def pdf_gumbel(u_values, theta, is_log=False):
     lip = ipsi_gumbel(u_values, theta, is_log=True)
     lnt = lsum(np.transpose(lip))
     alpha = 1.0 / theta
-    assert 0.0 < alpha, "0.0 < alpha: alpha = " + str(alpha) + \
+    assert 0.0 < alpha, "pdf_gumbel: 0.0 < alpha: alpha = " + str(alpha) + \
                         ", theta = " + str(theta)
-    assert alpha <= 1.0, "alpha <= 1.0: alpha = " + str(alpha) + \
+    assert alpha <= 1.0, "pdf_gumbel: alpha <= 1.0: alpha = " + str(alpha) + \
                          " theta = " + str(theta)
     lx = alpha * lnt
     ls = log_polyg(lx, alpha, d) - d * lx / alpha
